import { WaiterOptions } from "../waiter";
/**
 * @internal
 *
 * Validates that waiter options are passed correctly
 * @param options - a waiter configuration object
 */
export declare const validateWaiterOptions: <Client>(options: WaiterOptions<Client>) => void;
